;**********************************************************************
;   This program uses the PIC 16F628A to control a pulsing
;	device for electroplating
;**********************************************************************
;                                                                     *
;    Filename:	    electro02.asm                                     *
;    Date:                                                            *
;    File Version:                                                    *
;                                                                     *
;    Author:  		viduata		                                      *
;    Company:       nullspace labs                                    *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files Required: P16F628A.INC                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes: 
;	Here is the pin setup
;	PORTA:0		OUTPUT	Enable (Normally high)
;	PORTA:1		OUTPUT	RS (LCD)
;	PORTA:2		OUTPUT	R/W (LCD)
;	PORTA:3		INPUT	Forward-time incrementer button
;	PORTA:4		INPUT	Forward-time decrementer button
;	PORTA:5		INPUT	Reverse-time incrementer button
;	PORTA:6		INPUT	Reverse-time decrementer button
;	PORTA:7		OUTPUT	Forward/Reverse (1/0)
;	PORTB:0-7	OUTPUT	Parallel Data to the LCD

;	
;	Make sure that the LCD is setup to receive data through 8-pin
;	mode, using method in the datasheet
;**********************************************************************

	list      p=16f628A           ; list directive to define processor
	#include <p16F628A.inc>       ; processor specific variable definitions

	errorlevel  -302              ; suppress message 302 from list file


; Config notes. BOREN must be on, or else it can take 30 seconds for the 
;				device to reset when you turn it on or off.
;				MCLRE must be off, since we're using the pin as an input!!!
;				Timing is setup based on the 4MHz INTOSC
	__CONFIG   _CP_OFF & _DATA_CP_OFF & _LVP_OFF & _BOREN_ON & _MCLRE_OFF & _WDT_OFF & _PWRTE_ON & _INTOSC_OSC_NOCLKOUT 


;***** VARIABLE DEFINITIONS
forward_time			equ		020		; holds forward time, in milliseconds
reverse_time			equ		021		; holds reverse time, in milliseconds
current_status			equ		022		; keeps track of current flow.
										; 0=reverse. 1=forward
temp_holder				equ		023		; use whenever you need it
temp_working			equ		024		; holds W_REG data while handling interrupt
LCD_DATA_OUT			equ		025		; Data to display
LCD_CTRL				equ		026		; Control codes

LCD_wait_time			equ		028		; Wait time for LCD to process a command
LCD_wait_counter		equ		029		; Holder for counter
LCD_low_nibble			equ		02A		;
LCD_high_nibble			equ		02B		;
byte_to_convert			equ		02C		;
debounce_time			equ		027		; This variable holds debounce timing
debounce_inner			equ		02D		;
debounce_outer			equ		02E		;

forward_count			equ		02F		;
reverse_count			equ		030		;

;Below are new names for the pins in PORTA
RS						equ		1		; Control bits for LCD
RW						equ		2		;
EN						equ		0		;

FINC					equ		3		; forward incrementer
FDEC					equ		4		; forward decrementer
RINC					equ		5		; reverse incrementer
RDEC					equ		6		; reverse decrementer

OUT						equ		7		; actual h-bridge output
;**********************************************************************
	ORG     0x000             	; processor reset vector
	nop							;
	nop							;
	nop							;
	goto    MAIN              	; go to beginning of program

	

HANDLE_INTERRUPTS
	ORG     0x004             	; interrupt vector location
	movwf	temp_working		; save current working data
	call	H_BRIDGE_CTRL		; Do The Job!!!!
	movfw	temp_working		; Replace the working data
	call	CLEAR_INTS			; Else we're stuck here forever
	retfie						; return to wherever you were



MAIN
	call 	PIN_SETUP			; Added a lot of wasted time
	call	DEBOUNCE			; because the LCD seems a little slow
	call	DEBOUNCE			; on startups. When you update the
	call	DEBOUNCE			; setups, you have to give it time to
	call	SET_DEFAULTS		; run a few internal programs
	call 	INIT_LCD			;
	call	DEBOUNCE			; This whole routine takes
	call	UPDATE_LCD			; just under a second
	call	INTERRUPT_SETUP		; to set everything up nicely
	call 	TIMER0_SETUP		;
	call	TIMER0_INIT			;






MAIN_LOOP
	call	CHECK_BUTTONS		; check_buttons handles updates	
	goto	MAIN_LOOP			; Yes, this is the whole genius program loop
								

;---------------------------------------------------------------------------
; This,finally, is the program that flips the switch from high to low and
; back again.
; Routine : are we going forward or reverse?
; are we at zero, then flip-flop and reset
H_BRIDGE_CTRL
	btfsc	current_status,0	; test the direction bit
	goto	FORWARD_HANDLER		;
	goto	REVERSE_HANDLER		;
FORWARD_HANDLER
	decfsz	forward_count,f		;
	return						;
	goto	FLIP_REVERSE		;
REVERSE_HANDLER
	decfsz	reverse_count,f		;
	return						;
	goto	FLIP_FORWARD		;
FLIP_REVERSE					; Is it worth it? Let me work it.
	bcf		current_status,0	; set status=0=reverse
	bcf		PORTA, OUT			; flip the switch
	movfw	reverse_time		; copy the forward time into the
	movwf	reverse_count		; forward counter
	return						;
FLIP_FORWARD
	bsf		current_status,0	;
	bsf		PORTA, OUT			; flip the switch
	movfw	forward_time		;
	movwf	forward_count		;
	return						;
;---------------------------------------------------------------------------



;--------------------------------
; simplest sub ever
CLEAR_INTS
	BANKSEL	TMR0				;
	movlw	b'00001011'			; Empirically chosen counter value
	movwf	TMR0				; to create 993 microSecs per interrupt
	BANKSEL	INTCON				; This gives you 7 instructions to handle it
	bcf		INTCON, 2			; Clear int flag so we can move on.
	return						;





;---------------------------------------------------------------------------
; simple script to check the button status. note: they must be pulled low on
; the board or else who knows what values they will read?
CHECK_BUTTONS					; Subroutine will loop until an input changes
	BANKSEL	PORTA				; make sure we're not reading the TRISA settings
	btfsc	PORTA, FINC			;
	call	FORWARD_INC			;
	btfsc	PORTA, FDEC			;
	call	FORWARD_DEC			;
	btfsc	PORTA, RINC			;
	call	REVERSE_INC			;
	btfsc	PORTA, RDEC			;
	call	REVERSE_DEC			;
	return						;
FORWARD_INC
	incf	forward_time,f		;
	call	UPDATE_LCD			;
	call	DEBOUNCE			;
	return						;
FORWARD_DEC
	decf	forward_time,f		;
	call	UPDATE_LCD			;
	call	DEBOUNCE			;
	return						;
REVERSE_INC
	incf	reverse_time,f		;
	call	UPDATE_LCD			;
	call	DEBOUNCE			;
	return						;
REVERSE_DEC
	decf	reverse_time,f		;
	call	UPDATE_LCD			;
	call	DEBOUNCE			;
	return						;
;-------------------------------------------------------------------





; ------------------------------------------------------------------------------
; This sub takes whatever values are in forward_time and reverse_time,
; converts them to ascii, and then places them in the correct place
; on the LCD module
UPDATE_LCD						; Calculate offsets and update with Hex Values
	movlw	b'10000100'			; Move to display ram 0x04 (first char of FW time)
	movwf	PORTB				;
	bcf		PORTA, RS			;
	bcf		PORTA, EN			;
	bsf		PORTA, EN			;
	call 	LCD_WAIT			;
	swapf	forward_time, w		; high nibble first
	movwf	byte_to_convert		;
	call	CONVERT_VALUES		;
	call	WRITE_CHAR			;
	movfw	forward_time		; then low nibble
	movwf	byte_to_convert		;
	call	CONVERT_VALUES		;
	call	WRITE_CHAR			;
	movlw	b'11000100'			; Move to display ram 0x44 (first char of RV time)
	movwf	PORTB				;
	bcf		PORTA, RS			;
	bcf		PORTA, EN			;
	bsf		PORTA, EN			;
	call 	LCD_WAIT			;
	swapf	reverse_time, w		; high nibble first
	movwf	byte_to_convert		;
	call	CONVERT_VALUES		;
	call	WRITE_CHAR			;
	movfw	reverse_time		; then low nibble
	movwf	byte_to_convert		;
	call	CONVERT_VALUES		;
	call	WRITE_CHAR			;
	return						;



;-----------------------------------------------------------------
; Basic debouncer waits for about (x^2)*4 instructions
; where x = whatever you set debounce_timer to in defaults
DEBOUNCE
	movfw	debounce_time		;
	movwf	debounce_inner		;
	movwf	debounce_outer		;
INNER_DEBOUNCE
	decfsz	debounce_inner,f	;
	goto	INNER_DEBOUNCE		;
OUTER_DEBOUNCE
	decfsz	debounce_outer,f	;
	goto	INNER_DEBOUNCE		;
	return						;
;-------------------------------------------------------------------



;-------------------------------------------------------------------
; INIT_LCD is a big annoying sub that takes care of setting up
; the LCD and putting in the initial values
INIT_LCD						
	movlw	b'00000001'			; instruction to clear display
	movwf	PORTB				;
	bsf		PORTA,EN			; Set enable bit to high (normal setting)
	bcf		PORTA, RS			;
	bcf		PORTA, RW			;
	bcf		PORTA, EN			; Toggle enable bit
	bsf		PORTA, EN			; Back to normal mode
	call	LCD_WAIT			;
	movlw	b'00001100'			; Setup display: Display on, Cursor On, Blink On
	movwf	PORTB				;
	bcf		PORTA, RS			;
	bcf		PORTA, RW			;
	bcf		PORTA, EN			; Send command
	bsf		PORTA, EN			; Enable goes high again
	call	DEBOUNCE			; Needs extra wait time
	call 	LCD_WAIT			;
	movlw	b'00111000'			; Function set: 8 bit, 2 lines
	movwf	PORTB				;
	bcf		PORTA, EN			; Send command
	bsf		PORTA, EN			; Set enable to high again
	call	LCD_WAIT			;
	movlw	b'00000010'			; Return home
	movwf	PORTB				;
	bcf		PORTA, EN			;
	bsf		PORTA, EN			;
	call	DEBOUNCE			; Needs extra wait time
	call LCD_WAIT				;
	movlw	'F'					; Write out the Interface characters
	call	WRITE_CHAR			; "F: x32ms"
	movlw	':'					; "R: x01ms"
	call	WRITE_CHAR			;
	movlw	' '					; These are just initializing values
	call	WRITE_CHAR			; They are immediately updated with
	movlw	'x'					; actual values
	call	WRITE_CHAR			;
	movlw	'3'					;
	call	WRITE_CHAR			;
	movlw	'2'					;
	call	WRITE_CHAR			;
	movlw	'm'					;
	call	WRITE_CHAR			;
	movlw	'S'					;
	call	WRITE_CHAR			;
	movlw	b'10101000'			; Move to display ram 40 (second line)
	movwf	PORTB				;
	bcf		PORTA, RS			;
	bcf		PORTA, EN			;
	bsf		PORTA, EN			;
	call 	LCD_WAIT			;
	movlw	'R'					;
	call	WRITE_CHAR			;
	movlw	':'					;
	call	WRITE_CHAR			;
	movlw	' '					;
	call	WRITE_CHAR			;
	movlw	'x'					;
	call	WRITE_CHAR			;
	movlw	'0'					;
	call	WRITE_CHAR			;
	movlw	'1'					;
	call	WRITE_CHAR			;
	movlw	'm'					;
	call	WRITE_CHAR			;
	movlw	'S'					;
	call	WRITE_CHAR			;
	return						;
;-----------------------------------------------------------------------


;-----------------------------------------------------------------------------
; Just here to kill a little bit of time so that the LCD has
; time to pick up the instructions
LCD_WAIT						; Funtion to allow a few microseconds to pass
	movfw	LCD_wait_time		; 
	movwf	LCD_wait_counter	; Uses LCD_wait_time variable from defaults
LCD_WAIT_LOOP
	decfsz	LCD_wait_counter, f	;
	goto LCD_WAIT_LOOP			;
	return						;
;---------------------------------------------------------------------------

;--------------------------------------------------------------------------
; Very simply routine to take care of the overhead for writing to the
; next spot in the LCD Display Ram
WRITE_CHAR						; Takes char from W and writes it to LCD
	movwf	PORTB				;
	bsf		PORTA, RS			;
	bcf		PORTA, RW			;
	bcf		PORTA, EN			;
	bsf		PORTA, EN			;
	call	LCD_WAIT			;
	return						;
;--------------------------------------------------------------------------

	



;---------------------------------------------------------------------
; Set everything up in this section
SET_DEFAULTS
	bcf		STATUS, RP0			;
	movlw	0x34				; Default to 50ms ON
	movwf	forward_time		;
	movlw	0x12				; and 10ms OFF
	movwf	reverse_time		;
	movlw	0x80				; 10 ms to debounce
	movwf	debounce_time		;
	movlw	0x30				; minimum 60 instructions
	movwf	LCD_wait_time		; of wait time before sending next LCD instruction
	banksel	PORTA				;
	clrf	PORTA				;
	movlw	b'00000001'			; I have completely forgotten what this is for
	movwf	PORTB				; I should probably erase these two lines
	return						;
;---------------------------------------------------------------------------



;----------------------------------------------------------
; needed for the timing routines
TIMER0_SETUP
	BANKSEL	OPTION_REG			;
	bcf		OPTION_REG, T0CS	; TMR0 uses internal clock
	bcf		OPTION_REG, PSA		; Prescaler assigned to Timer0
	bsf		OPTION_REG, PS0		; Sets prescaler to 001, or 1/4
	bcf		OPTION_REG, PS1		;
	bcf		OPTION_REG, PS2		; Now TMR0 is set up
	BANKSEL PIE1				;
	clrf	PIE1				;
	BANKSEL	INTCON				;
	bsf		INTCON, 5			; sets the Timer0 Interrupt flag
	BANKSEL	PORTA				;
	return						;
;--------------------------------------------------------------

;-----------------------------------------------------------
;Default values for TIMER0
TIMER0_INIT
	movlw	b'11111111'			; we want the difference between FF
	subwf	forward_time, w		; and the amount of time to wait
	BANKSEL	TMR0				;
	movwf	TMR0				; set timer
	bsf		current_status, 0	; put us in forward mode
	return						;
;---------------------------------------------------------


;--------------------------------------------------------------
; default pin configuration - this may change if i figure out
; the 4-pin com method for the LCD
PIN_SETUP
	movlw	b'00000111'			; By default, the first four inputs of PORTA
	movwf	CMCON				; are set as analog comparators on this chip
								; We need to make this correction to allow
								; normal input on PORTA<0:3>
	movlw	b'00000000'			; Pin setup is described
	BANKSEL	TRISB				; in the notes section
	movwf	TRISB				; PORTB is all output
	movlw	b'01111000'			;
	movwf	TRISA				; PORTA has 4 inputs and 4 outputs
	return						;
;-----------------------------------------------------------------------






;-----------------------------------------------------------------------
; this sets up our interrupts so that a TMR0 overflow takes us to the 
; interrupt vector, where we will switch the direction of output
INTERRUPT_SETUP
	BANKSEL	INTCON				;
	clrf	INTCON				; set all to zeroes
	movlw	b'10100000'			;
	movwf	INTCON				; set up TMR0 overflow interrupt
	return						;
;------------------------------------------------------------------------




;----------------------------------------------------------------------------------
;	CONVERT_HEX_TO_ASCII changes the low nibble of a byte
;	into an ascii code for the character
CONVERT_VALUES
	movlw	b'00001111'			;
	andwf	byte_to_convert,f	; Changes the high nibble to zeroes
	movlw	0					; I'm not sure if 0-0=0 trips the zero flag
	subwf	byte_to_convert,w	;
	btfsc	STATUS, Z			;
	retlw	'0'					;
	movlw	1;					
	subwf	byte_to_convert,w	;
	btfsc	STATUS, Z			;
	retlw	'1'					;
	movlw	2					;
	subwf	byte_to_convert,w	;
	btfsc	STATUS, Z			;
	retlw	'2'					;
	movlw	3					;
	subwf	byte_to_convert,w	;
	btfsc	STATUS, Z			;
	retlw	'3'					;
	movlw	4					;
	subwf	byte_to_convert,w	;
	btfsc	STATUS, Z			;
	retlw	'4'					;
	movlw	5					;
	subwf	byte_to_convert,w	;
	btfsc	STATUS, Z			;
	retlw	'5'					;
	movlw	6					;
	subwf	byte_to_convert,w	;
	btfsc	STATUS, Z			;
	retlw	'6'					;
	movlw	7					;
	subwf	byte_to_convert,w	;
	btfsc	STATUS, Z			;
	retlw	'7'					;
	movlw	8					;
	subwf	byte_to_convert,w	;
	btfsc	STATUS, Z			;
	retlw	'8'					;
	movlw	9					;
	subwf	byte_to_convert,w	;
	btfsc	STATUS, Z			;
	retlw	'9'					;
	movlw	0xA					;
	subwf	byte_to_convert,w	;
	btfsc	STATUS, Z			;
	retlw	'A'					;
	movlw	0xB					;
	subwf	byte_to_convert,w	;
	btfsc	STATUS, Z			;
	retlw	'B'					;
	movlw	0xC					;
	subwf	byte_to_convert,w	;
	btfsc	STATUS, Z			;
	retlw	'C'					;
	movlw	0xD					;
	subwf	byte_to_convert,w	;
	btfsc	STATUS, Z			;
	retlw	'D'					;
	movlw	0xE					;
	subwf	byte_to_convert,w	;
	btfsc	STATUS, Z			;
	retlw	'E'					;
	movlw	0xF					;
	subwf	byte_to_convert,w	;
	btfsc	STATUS, Z			;
	retlw	'F'					;
	retlw	'X'					;
;---------------------------------------------------------------




	END                       ; directive 'end of program'


;------------------------------------------------------------------
; In a future version, I'll use this to hold the last saved values
;INIT_EEPROM
;	ORG	0x2100
;	DE	0x00, 0x01, 0x02, 0x03
;----------------------------------------------------